import java.util.*;


public final class ElCombinator
{
	private LinkedList theList	;
	static ElCombinator theInstance;

	public static void main(String[] args)
	{
		theInstance = new ElCombinator();
		theInstance.addEL(1);
		theInstance.addEL(2);
		theInstance.addEL(3);
		theInstance.printList();
		System.out.println("Reduced = " + theInstance.getListEL((LinkedList)theInstance.theList.clone()));
		System.out.println(" ");
		theInstance.printList();
	}

	public ElCombinator()
	{
		theList = new LinkedList();
	}

	public void printList()
	{
		ListIterator li = theList.listIterator(0);
		System.out.println("List; ");
		while(li.hasNext() == true)
			System.out.println(li.next() + ", ");
	}

	public void addEL(int el)
	{
		theList.add(new Integer(el));
	}

	// the logic of the loops is as follows:

	// note that the term "reduce" is used to describe the process of comparing 2 EL/CR values and
	// finding that they are either equal (matched pair) or that one is 2 less than the other (mixed pair)
	// matched pairs yield a single value 2 higher than their current value
	// mixed pairs yield a value 1 higher than the higher value in the pair

	// the term "reduce" is used because in these cases the newly produced value is substituted into
	// the list for one half of the pair, and the other half is deleted
	// hence each time this is accomplished, the list size "reduces" by one

	// therefore a successfully reduced list is a single value, and an irreducible list is one wherein no
	// two elements are equal, and no element is 2 less than another

	// [5, 9, 14] is an example of an irreducible list

	// we start with the head of the list as "current"
	// we also return to this state whenever we successfully reduce the list
	// when dealing with each "current" we compare it with every folloing element
	// if we cannot find any element to reduce with the current one, we increment the current element
	// then we try to reduce with that element and all elements which follow it
	// if we reach the point where we have failed to reduce using all but one element as "current" then
	// the list is completely irreducible

	// we currently have no concept for reducing this kind of list to a single value

	// i suspect that there is some advantage to sorting the list in ascending order
	// however this would need to be investigated, since the advantage would only
	// come if the list were re-sorted on every reduction; possibly a very high overhead

	public int getListEL(LinkedList list)
	{
		int temp;
		int current;
		int match;
		boolean reduce;
		boolean irreducible = false;

		// this loop is the "per reduction" loop
		// we start at the beginning of the list for each iteration
		while(irreducible == false && list.size() > 1)
		{
			// SORT THE LIST HERE!!
			current = 0;
			reduce = false;
			// for each current...
			while(reduce == false && current < list.size())
			{
				match = current + 1;
				// this is the "for each other element" loop
				while(reduce == false && match < list.size())
				{
					temp = getPairEL(((Integer)list.get(current)).intValue(), ((Integer)list.get(match)).intValue());
					if(temp != -1)
					{
						reduce = true;
						list.remove(match);
						list.set(current, new Integer(temp));
					}
					else
					{
						match++;
					}
				}
				// if we get here and reduce is false, then current cannot be reduced in combination
				// with any element following it
				if(reduce == false)
					current++;
			}

			// if we get here, and reduce is false, no combination of elements can be reduced!
			if(reduce == false)
				irreducible = true;
		}

		if(list.size() > 1)
			return -1; // the list is irreducible at some point
		else
			return ((Integer)list.get(0)).intValue(); // our reduced list
	}



	// we are defining EL -1 as being "cannot reduce"
	// this still needs specific case handling for x <= 1
	// see DMG pp101 - would need a case statement i think

	public int getPairEL(int a, int b)
	{
		int x = a, y = b;
		if(a < b)
		{
			x = b;
			y = a;
		}

		if(x == y)
		{
			return x + 2;
		}
		else if((x - 2) == y)
		{
			return x + 1;
		}
		else
		{
			return -1;
		}
	}
}